#!/usr/bin/perl -w

########################################################
# 
# cutSequences.pl
#    part of the NcDNAlign alignment pipeline, step (1)
#
# Authors: Jana Hertel, Dominic Rose
#
# Contact: jana@bioinf.uni-leipzig.de
#          dominic@bioinf.uni-leipzig.de
# http://www.bioinf.uni-leipzig.de/Software/NcDNAlign/
#
########################################################

use strict;
use NcDNAlign;
use Bio::SeqIO;
use Data::Dumper;
use Getopt::Long;
use Pod::Usage;

# Keep in mind: BioPerl does not handle circular sequences!!!

my ($opt_conf, $opt_gbk, $opt_name, $opt_gbk_keys, $opt_cut, $opt_minLen, $opt_help, $opt_version, $opt_man) = 
('', '', '', '', '', '', 0, '', '', '');

GetOptions(
	"c|conf:s"	=> \$opt_conf,
	"gbk:s"		=> \$opt_gbk,
	"n|name:s"	=> \$opt_name,
	"gbk_keys:s"	=> \$opt_gbk_keys,
	"cut:s"	 	=> \$opt_cut,
	"min_aln_len:s"	=> \$opt_minLen,
	"h|help!"	=> \$opt_help,
	"v|version!"	=> \$opt_version,
	"man!"		=> \$opt_man
) or pod2usage(2);

pod2usage(1) if ($opt_help);
pod2usage(-exitstatus => 0, -verbose => 2) if $opt_man;

if($opt_version) {
	print "This is cutSequences.pl, part of the NcDNAlign package, version: $VERSION.\n$url\n";
	exit;
}

# ------ Start main ----------
my ($path, @gbk_keys, %gbk_keys);

if($opt_conf) {
	if($opt_gbk) {
		print STDERR "You can not run the program using -conf and -gbk both!\n";
		pod2usage(2);
	}
	if($opt_name) {
		print STDERR "You can not run the program using -conf and -name both!\n";
		pod2usage(2);
	}
	if($opt_gbk_keys) {
		print STDERR "You can not run the program using -conf and -gbk_keys both!\n";
		pod2usage(2);
	}
	if($opt_cut) {
		print STDERR "You can not run the program using -conf and -cut both!\n";
		pod2usage(2);
	}
	if($opt_minLen) {
		print STDERR "You can not run the program using -conf and -min_aln_len both!\n";
		pod2usage(2);
	}

	# EITHER

	loadConfig($opt_conf);	

	foreach my $name (sort keys %ORG) {

	  opendir(SRC, $C{source_dir}) or die "$!\nDIR: $C{source_dir}";
	  my @files = grep {-f $C{source_dir}."/".$_ && $_ =~ /$name.*\.gbk/} readdir(SRC);
	  closedir(SRC);
	  
	  # more than 1 gbk file
	  if(scalar @files > 1) {
	    die $die_msg."You have more than one .gbk-file for organism: $name";
	    exit(1);
	  }
	  # no gbk file, ensure fasta-file
	  if(scalar @files == 0) {
	    opendir(SRC, $C{source_dir}) or die "$!\nDIR: $C{source_dir}";
	    my @fa_files = grep {-f $C{source_dir}."/".$_ && $_ =~ /$name\.fa/} readdir(SRC);
	    closedir(SRC);

	    my $qt_fa = scalar @fa_files;
	    if($qt_fa !=1) {
	      die $die_msg."You do not have one .*gbk or one .*fa file for organism: '$name'\n".Dumper(@fa_files);
	    }
	    else {
	      printf STDOUT "SUCCESS: <= %s\n",$fa_files[0];
	      $path = $C{source_dir}."/".$fa_files[0];      
	      changeHeader($path,$name);
	    }
	  }
	  else {
	    printf STDOUT "SUCCESS: <= %s\n",$files[0];
	    $path = $C{source_dir}."/".$files[0];
	    hashKeys($C{gbk_keys});
	    parseGenBank($path, $name);
	  }
	}
}
else { # one single file!
	if(!$opt_gbk) {
		print STDERR "You did not specify -conf, this implies mode 2, thus, -gbk is required!\n";
		pod2usage(2);
	}
	if(!$opt_name) {
		print STDERR "You did not specify -conf, this implies mode 2, thus, -name is required!\n";
		pod2usage(2);
	}
	if(!$opt_gbk_keys) {
		print STDERR "You did not specify -conf, this implies mode 2, thus, -gbk_keys is required!\n";
		pod2usage(2);
	}
	if(!$opt_cut) {
		print STDERR "You did not specify -conf, this implies mode 2, thus, -cut is required!\n";
		pod2usage(2);
	}
	if(!$opt_minLen) {
		print STDERR "You did not specify -conf, this implies mode 2, thus, -min_aln_len is required!\n";
		pod2usage(2);
	}	
	
	
	printf STDOUT "SUCCESS: <= %s\n", $opt_gbk;
	hashKeys($opt_gbk_keys);
	parseGenBankSingle($opt_gbk, $opt_name);
}
# ------ End main ------------

# ------ Start functions -----

sub hashKeys {
	my ($string) = @_;
	
	@gbk_keys = split /\;/,$string;
	foreach my $k (@gbk_keys) {
		$gbk_keys{$k} = 1;
	}
}

sub changeHeader {
  my ($file, $name) = @_;
  
  my $out_f = $C{source_dir}."/".$ORG{$name}{ncregions};
  open(OUT, ">", $out_f) or die $die_msg.$!;
  printf "         => write sequences to %s\n",$out_f;
  
  my $liste = $out_f;
  $liste .= ".lst";
  printf "         => write scaffold/chr. mapping to %s\n",$liste;
  my ($head, $seq, $len, $n, $id);
  $seq = "";
  $n = 0;

  my %liste;
  open(IN, $file) or die $die_msg.$!;
  while(<IN>) {
    if($_ =~ /^>(\S+)/) {
      $n++;
      $id = $1;
      $len = length $seq;
      if($len > 0) {
	$head .= "1:".$len.":+";
	printf OUT "%s\n%s\n",$head,$seq;
      }
      $head = ">".$name."_".$n.":";
      $seq = "";
      $liste{$name."_".$n} = $id;
#      printf A "%s_%d\t%s\n",$name,$n,$id;
    }
    else {
      chomp($_);
      $seq =~ s/\s+$//;
      $seq .= $_;
    }
    
  }

  open(A, ">", $liste) or die $die_msg.$!;
  foreach my $key (keys %liste) {
    printf A "%s\t%s\n",$key, $liste{$key};
  }

  close(A);
  close(IN);
  close(OUT);
}

sub parseGenBank {
  my ($file,$name) = @_;

  my $in = new Bio::SeqIO(-format => 'genbank', -file => $file);
  my ($length, $sequence,$i,$count);
  
  $count = 0;

  my $out_f = $C{source_dir}."/".$ORG{$name}{ncregions};
  open(OUT, ">", $out_f) or die $die_msg."$out_f: $!";
  printf "         => write sequences to %s\n",$out_f;

  my $liste = $out_f;
  $liste .= ".lst";
  printf "         => write scaffold/chr. mapping to %s\n",$liste;

  my %liste;
  while ( my $seq = $in->next_seq ) {
    $count++;
    my @regions;
    $length = $seq->length();
    $regions[0] = "x";
    for($i=1; $i<=$length; $i++) { $regions[$i] = 1; }
    $sequence = $seq->seq();

    my (@toCut,$c,$start,$end);
    foreach my $f ( $seq->top_SeqFeatures ) {
      if(exists $gbk_keys{$f->primary_tag}) {
	  $start = scalar $f->start;
	  $end = scalar $f->end;
	  if($end < $start) {
	    die $die_msg."The coords are fucked up in your .gbk file";
	  }
	  for($i= $start; $i<=$end; $i++) {
	    $regions[$i] = 0;
	  }
      }
    }

    my ($s,$l,$sold,$subseq);
    $s = 0;
    for($i=1; $i<=$length; $i++) {
      if($regions[$i] == 0 && $s == 0) { next; }
      elsif($regions[$i] == 1 && $s == 0) {
	  $s = $i;
	  $sold = $s;
	  $l = 1;
	}
      elsif($regions[$i] == 1 && $s > 0) {
	$l++;
	if($i == $length) {
	  if($l > $C{minAlnLen}) {
	    $subseq = substr($sequence, $s-1, $l);
	    printf OUT ">%s\_%d:%d:%d:+\n%s\n",$name,$count,$s,$l,$subseq;
	    $liste{$name."_".$count} = $seq->display_id();
	    #printf A "%s_%d\t%s\n",$name,$count,$seq->display_id();
	  }
	}
      }
      elsif($regions[$i] == 0 && $s > 0) {
	if($l > $C{minAlnLen}) {
	  $subseq = substr($sequence, $s-1, $l);
	  printf OUT ">%s\_%d:%d:%d:+\n%s\n",$name,$count,$s,$l,$subseq;
	  $liste{$name."_".$count} = $seq->display_id();
	  # printf A "%s_%d\t%s\n",$name,$count,$seq->display_id();
	}
	$s = 0;
      }
    }
  }
  close(OUT);

  open(A, ">", $liste) or die $die_msg.$!;
  foreach my $key (keys %liste) {
    printf A "%s\t%s\n",$key, $liste{$key};
  }
  close(A);

}



# Variant for a single gbk file
sub parseGenBankSingle {
	my ($file, $name) = @_;

	my $in = new Bio::SeqIO(-format => 'genbank', -file => $file);
	my ($length, $sequence, $i, $count);

	$count = 0;

	open(OUT, ">", $opt_cut) or die $die_msg."$opt_cut: $!";
	printf "         => write sequences to %s\n", $opt_cut;

	my $liste = $opt_cut;
	$liste .= ".lst";
	printf "         => write scaffold/chr. mapping to %s\n", $liste;

	my %liste;
	while ( my $seq = $in->next_seq ) {
		$count++;
		my @regions;
		$length = $seq->length();
		$regions[0] = "x";
		for($i=1; $i<=$length; $i++) { $regions[$i] = 1; }
		$sequence = $seq->seq();

		my (@toCut,$c,$start,$end);
		foreach my $f ( $seq->top_SeqFeatures ) {
			if(exists $gbk_keys{$f->primary_tag}) {
				
				$start = scalar $f->start;
				$end = scalar $f->end;
				
				if($end < $start) {
					die $die_msg."The coords are fucked up in your .gbk file";
				}
				for($i=$start; $i<=$end; $i++) {
					$regions[$i] = 0;
				}
			}
		}

		my ($s,$l,$sold,$subseq);
		$s = 0;
		for($i=1; $i<=$length; $i++) {
			#print $regions[$i];
			if($regions[$i] == 0 && $s == 0) { next; }
			elsif($regions[$i] == 1 && $s == 0) {
				$s = $i;
				$sold = $s;
				$l = 1;
			}
			elsif($regions[$i] == 1 && $s > 0) {
				$l++;
				if($i == $length) {
					if($l > $opt_minLen) {
						$subseq = substr($sequence, $s-1, $l);
						printf OUT ">%s\_%d:%d:%d:+\n%s\n",$name,$count,$s,$l,$subseq;
						$liste{$name."_".$count} = $seq->display_id();
						#printf A "%s_%d\t%s\n",$name,$count,$seq->display_id();
					}
				}
			}
			elsif($regions[$i] == 0 && $s > 0) {
				if($l > $opt_minLen) {
					$subseq = substr($sequence, $s-1, $l);
					printf OUT ">%s\_%d:%d:%d:+\n%s\n",$name,$count,$s,$l,$subseq;
					$liste{$name."_".$count} = $seq->display_id();
					#printf A "%s_%d\t%s\n",$name,$count,$seq->display_id();
				}
				$s = 0;
			}
		}
	}
	close(OUT);

	open(A, ">", $liste) or die $die_msg.$!;
	foreach my $key (keys %liste) {
		printf A "%s\t%s\n",$key, $liste{$key};
	}
	close(A);
}
# ------ End functions --------

__END__

=head1 NAME

C<cutSequences.pl> - part of the NcDNAlign alignment pipeline, step (1)

Extracts (sub)sequences out of sequence files and writes them out in NcDNAlign standardized FASTA format
(supported input file formats: FASTA, GenBank).

=head1 SYNOPSIS

cutSequences.pl [options_mode_1 XOR options_ mode_2]
 
There are two ways of running the program: 

=over 8

=item Mode 1:

EITHER you specify the a configuration file and the program runs for a whole screen, 

=item Mode 2:

OR you specify a gbk file, the organisms name, the disturbing gbk feature keys and an outfile to cut one single gbk file.

=back

=head1 OPTIONS

=head2 Mode 1:

=over 8

=item B<-c, --conf> FILE

Path to central NcDNAlign configuration file,
inhibits -gbk, -name, -gbk_keys, -cut, -min_aln_len

=back

=head2 Mode 2:

=over 8

=item B<--gbk> FILE

Path to a single gbk file.

=item B<-n, --name> xyz

The 3-letter name of the organism that belongs to the gbk file

=item B<--gbk_keys> "foo;bar"

Specify gbk feature keys that should be cutted out of the gbk file. List them comma separated! Reference of GenBank feature keys: http://www.ncbi.nlm.nih.gov/collab/FT/#7.3

=item B<--cut> FILE

Path to a file taking the output in NcDNAlign customized fasta format. Cutted regions will be stored there (e.g. *.cut.fa).

=item B<--min_aln_len> INT

Minimal required alignment length.

=item B<-v, --version>

Prints version information and exits.

=item B<-h, --help>

Prints a short help message and exits.

=item B<--man>

Prints a detailed manual page and exits.

=back

=head1 DESCRIPTION

cutSequences.pl optionally cuts out specific regions from genome files resulting in FASTA formatted files that either contain subsequences or the complete genomic sequence with appropriate headers. 

If it is requested to get alignments of all possible regions of applied genomes and nothing should be cutted, just deliver an empty string for the option C_GBK_KEYS in the config file or on the -gbk_keys option at the command line. Getting rid of disturbing elements as earliest as possible directly speeds up subsequent BLAST searches and further alignment procedures due to shorter sequences.

However, one must be familiar with the GenBank feature keys documented at http://www.ncbi.nlm.nih.gov/collab/FT/#7.3 to ensure that no desired loci are unintentionally cut out of the genomes (e.g. tRNAs can appear at exons, so dropping exons would cause to loose tRNAs).

=head1 EXAMPLES

=over 8

=item Mode 1 (whole screen):

$ cutSequences.pl -c config-file.cfg

=item Mode 2 (cutting single file):

$ cutSequences.pl -gbk Eco.gbk -name Eco -gbk_keys "CDS;repeat_region" -cut Eco.cut.fa

=back

=head1 AUTHORS

 Dominic Rose (dominic@bioinf.uni-leipzig.de)
 Jana Hertel  (jana@bioinf.uni-leipzig.de)

=head1 AVAILABILITY

http://www.bioinf.uni-leipzig.de/Software/NcDNAlign/

=cut
