#!/usr/bin/perl -w

########################################################
# 
# realign.pl
#    part of the NcDNAlign alignment pipeline, step (4)
#
# Author: Dominic Rose
#
# Contact: dominic@bioinf.uni-leipzig.de
# http://www.bioinf.uni-leipzig.de/Software/NcDNAlign/
#
########################################################

use NcDNAlign;
use strict;
use Getopt::Long;
use Math::Round;
use Pod::Usage;

my ($opt_conf, $opt_out, $opt_help, $opt_version, $opt_man, $opt_s) = ('', 0, '', '', '', 0);

GetOptions(
	"c|conf=s" 	=> \$opt_conf,
	"o|out:i" 	=> \$opt_out,
	"h|help"	=> \$opt_help,
	"v|version"	=> \$opt_version,
	"man"		=> \$opt_man,
	"s|silent:i"	=> \$opt_s
) or pod2usage(2);

pod2usage(1) if ($opt_help || $opt_s !~ /^[1|0]$/);
pod2usage(-exitstatus => 0, -verbose => 2) if $opt_man;

if($opt_version) {
	print "This is cutSequences.pl, part of the NcDNAlign package, version: $VERSION.\n$url\n";
	exit;
}

if(!$opt_conf){
	pod2usage(1);
}

if($opt_out !~ /^[0|1]$/) {
	pod2usage(1);
}


# ------ Start ----------
loadConfig($opt_conf);

my $resDir 		= "$C{p_dir}/blast";
my %data; 				# all data of all bla[s]t result files together in one hash
my %line; 				# a hash containing the data of a single line	
my $bedFile 		= "$C{p_dir}/gwAln.bed"; # gw = genomewide
my %bed;
my @Files; 				# bla[s]t files
my $qtOverall		= 0;
my $out_dir		= '';
my $qtOrganisms 	= keys %ORG;	# Are there enough sequences for a valid alignment?
my $maxDigitSize 	= 12;		# how big are genome sizes? Number of positions of the size
my $max_step		= 1000;		# do not put more than 1000 alignments into one directory
my $qtSubDir 		= 1; 		# the directory name, we start with 1
my $step		= 1;		# a first alignment
my %subs; 				# which subjects are in an alignment?
my %seenPos = ();
my $tmpFile = $C{source_dir}."/t.NcDNAlign";


main();

sub main {
	
	# check for output dir
	$out_dir = $C{aln_dir};
	if( !(-d $out_dir) ) {
		print "creating out_dir: '$out_dir'\n" if($opt_out==1);
		`mkdir $out_dir`;
	}
	
	opendir(DIR, $resDir) or die $die_msg."Could not open directory '$resDir'!";
	@Files = readdir(DIR);
	close DIR;

	hashInputNew();
	print "performing alignment ...\n" if($opt_s==0);
	do_it();
	
	open(BED, ">$bedFile") or die $die_msg."Could not open file '$bedFile'!";
	# write bed file
	foreach my $chromosome (sort {$a<=>$b} keys %bed) {
		foreach my $start (sort {$a<=>$b} keys %{$bed{$chromosome}}) {
			my @e = split /;/, $bed{$chromosome}{$start};
			die $die_msg."Exception: Wrong bed format!" if(scalar(@e) != 5);
			print BED "$e[2]\t$e[0]\t$e[1]\tgwAln_$e[4]\t.\t$e[3]\n";
		}
	}	
	close BED;
	
	# remove tmp files
	unlink ($tmpFile);	
	
	if($opt_s==0) {
		print "\nFinished => Extracted $qtOverall alignments.\n";
		print "BED file of genomewide alignment positions => '$bedFile'\n\n"; 	
	}
}



sub do_it {
	my %thisData;
		
	foreach my $query (sort keys %data) {	
		
		my @posKeys = sort keys %{$data{$query}};
		my ($minQS, $maxQE) = (0,0);
		%subs = ();		
		
		foreach my $qPos (@posKeys) {

			if(keys %seenPos == 0) { # initialize
				$seenPos{$qPos} = 1;
			}
						
			my @p = split (/\:/, $qPos);
									
			# consistency check
			die $die_msg."Wrong position format: '$qPos'!" if($p[0]>$p[1]);
								
			foreach my $subject (sort keys %{$data{$query}{$qPos}}) {
				
				#print "$subject=$data{$query}{$qPos}{$subject}\n";
				
				%thisData = %{readInputLine($data{$query}{$qPos}{$subject})};
				print "q=$query    pos=$qPos   s=$subject   eV=$thisData{eV}\n" if($opt_out==1);
				die $die_msg."Empty entry: q='$query', pos='$qPos', s='$subject', eV='$thisData{eV}'" if($query eq '' || $qPos eq '' || $subject eq '' || $thisData{eV} eq '');
				# if curQuery overlaps with lastQuery, then link them
				# else make two out of them 
			}
			
			# be careful:
			# there is always sS>sE, because of readInputLine()
			# negative strand is indicated by subStrand
			
			# initialize query positions
			$minQS=$p[0] if($minQS==0);
			$maxQE=$p[1] if($maxQE==0);
						
			if( olWithAllSeen($p[0], $p[1], %seenPos)  ) {
				# get minQS and maxQE
				if($p[0]<$minQS) {
					$minQS = $p[0]; # better start?
				}
				if($p[1]>$maxQE) {
					$maxQE = $p[1]; # better end?
				}
				
				addThisSubject($query, $p[0], $p[1], $thisData{eV}, $thisData{sS}, $thisData{sE});
			}
			else {				
				# no overlap, a new alignment
				# write out the current read alignment
				weHaveAnAln($query, $minQS, $maxQE);
				
				($minQS, $maxQE) = ($p[0],$p[1]);
				%subs = ();
				%seenPos = ();
				addThisSubject($query, $p[0], $p[1], $thisData{eV}, $thisData{sS}, $thisData{sE});
			}
			
			#$lastPos = $qPos;
			$seenPos{$qPos} = 1;
		}
		
		# don't forget the last one !!!!!!
		weHaveAnAln($query, $minQS, $maxQE);				

		print "\n" if($opt_out==1);;
	}	
}

# Do we have an overlap of the current pos with all other pos in the alignment?
sub olWithAllSeen {
	my ($thisStart, $thisEnd, %seenPos) = @_;
	
	my $ret = 1;
	
	foreach my $k (sort keys %seenPos) {

		my @p = split (/\:/, $k);
		if(overlapping($thisStart, $thisEnd, $p[0], $p[1])) {
			print "\toverlap $thisStart, $thisEnd, $p[0], $p[1]\n" if($opt_out==1);
		}
		else {
			print "\tNo overlap for $thisStart, $thisEnd, $p[0], $p[1]!\n" if($opt_out==1);
			$ret=0;
			last;
		}
	}

	return $ret;
}

# what happens if an aln is recognised
sub weHaveAnAln {
	my ($query, $minQS, $maxQE) = @_;	
	
	if($opt_out==1) {
		print "--- setting up alignment ---\n";
		print "minQS=$minQS, maxQE=$maxQE\n";
	}
	
	foreach my $k (sort keys %subs) {
		print "s\t$k  qS=$subs{$k}{qS}  qE=$subs{$k}{qE}  eV=$subs{$k}{eV}  sub=$subs{$k}{data}  sS=$subs{$k}{sS}  sE=$subs{$k}{sE}\n" if($opt_out==1);
		# ensuring quality
		die $die_msg."At least one entry is empty: qS=$subs{$k}{qS}  qE=$subs{$k}{qE}  eV=$subs{$k}{eV}  sub=$subs{$k}{data}  sS=$subs{$k}{sS}  sE=$subs{$k}{sE}" if($subs{$k}{qS} eq '' || $subs{$k}{qE} eq '' || $subs{$k}{eV} eq '' || $subs{$k}{data} eq '' || $subs{$k}{sS} eq '' || $subs{$k}{sE} eq '');
	}
	
	# extract the alignment
	my $qtMembers = (keys %subs)+1; 
	# BUG: 28/06/2007: The subject hash %subs contains 1 entry for each blast hit
	# Thus in case of 2-way alignments it has 1 entry. The +1 is necessary because two sequences are participating. 
	#print "qtMembers=$qtMembers";
	
	# but only if it contains enough sequences
	if( $qtMembers >= $C{min_seq} ) {
		$qtOverall++;
		
		# do not put more than $max_step alignment files in one directory
		if($step>$max_step) {
			$step=1;
			$qtSubDir++;
		}
		my $pathSubDir 	= $out_dir."/".$qtSubDir;
		my $outFileFa	= "$pathSubDir\/$qtOverall\.fa";
				
		if(-d $pathSubDir ) {
			
		}
		else {
			print "creating subDir: '$pathSubDir'\n" if($opt_out==1);
			`mkdir $pathSubDir`;
		}
		
		open(OUT, "> $outFileFa") or die $die_msg."Could not open file '$outFileFa'";
			
		# get query sequence
		# remove the leading 0s prior needed for sorting
		my $reducedQuery = removeNulls($query);
		
		my ($refName, $refStart, $strand, $strandString, $refRegionLen) = ('',0,'', '+', 0);
			
		# example: Prochlorococcus_marinus_str_MIT_9312:1051264:1629:+
		if($reducedQuery =~ /^(.+)\:(\d+)\:(\d+)\:(.)/) {
			$refName 	= $1;
			$refStart 	= $2;
			$refRegionLen 	= $3; 
		}
		else {
			die $die_msg."reducedQuery=$reducedQuery has wrong format!";
		};
		
		# remove sorting-zeros
		$minQS =~ s/^0*//g;
		$maxQE =~ s/^0*//g;
		
		# add flanking region to query
		$minQS = addFlankingStart($minQS);
		$maxQE = addFlankingEnd($maxQE, $refRegionLen);		
		
		# always cut query in forward direction
		my $cmd = "fastacmd -d $C{source_dir}/$ORG{$C{target_name}}{ncregions} -s $reducedQuery -L $minQS,$maxQE 2> $tmpFile";
			
		print "$outFileFa\n$cmd\n" if($opt_out==1);
		my $back = `$cmd`;
		if(-e "$tmpFile") {
			open(TMPIN, "< $tmpFile") or die $die_msg."Cant open tmp file $tmpFile (during setting up '$outFileFa') - Maybe there is something wrong with the sys call: '$cmd'?";
			my @tmpdata = <TMPIN>;
			foreach my $line (@tmpdata) {
				die $die_msg."Error occured:$line" if($line =~/ERROR/);
			}
			close(TMPIN);
		}
		$back =~ s/^>.+//g; # remove weird fastacmd error!
		
		print OUT ">$reducedQuery:hitStart=$minQS:hitEnd=$maxQE:+";
		print OUT "$back";
		
		# collect data for bed
		my $bedStart = $refStart+$minQS-1;
		my $bedEnd = $refStart+$maxQE-1;
		$refName =~ /\_(\d+)/;
		$bed{$1}{$bedStart}="$bedStart;$bedEnd;$refName;+;$reducedQuery\_file=$qtOverall";
		
		# get subject sequences
		foreach my $k (sort keys %subs) {
			$strand='';
			$strandString='+';
			my $reducedSubject = removeNulls($subs{$k}{data});
			
			my $subRegionLen = '';
			if($reducedSubject =~ /^(.+)\:(\d+)\:(\d+)\:(.)/) {
				$subRegionLen 	= $3; 
			}
			else {
				die $die_msg."reducedSubject=$reducedSubject has wrong format!";
			};
			
			if( $subs{$k}{subStrand} eq 0 ) {
				$strand = "-S2";
				$strandString='-';
			}
			
			# remove sorting-zeros
			$subs{$k}{sS} =~ s/^0*//g;
			$subs{$k}{sE} =~ s/^0*//g;
			
			# add flanking region to subject
			$subs{$k}{sS} = addFlankingStart($subs{$k}{sS});
			$subs{$k}{sE} = addFlankingEnd($subs{$k}{sE}, $subRegionLen);			
			
			$cmd = "fastacmd $strand -d $C{source_dir}/$ORG{$k}{ncregions} -s $reducedSubject -L $subs{$k}{sS},$subs{$k}{sE} 2> $tmpFile";
			print "$cmd\n" if($opt_out==1);
			$back = `$cmd`;
			if(-e "$tmpFile") {
				open(TMPIN, "< $tmpFile") or die $die_msg."Cant open tmp file $tmpFile";
				my @tmpdata = <TMPIN>;
				foreach my $line (@tmpdata) {
					die $die_msg."Error occured:$line" if($line =~/ERROR/);
				}
				close(TMPIN);
			}
			$back =~ s/^>.+//g; # remove weird fastacmd error!
										
			my $outHead = ">$reducedSubject:hitStart=$subs{$k}{sS}:hitEnd=$subs{$k}{sE}:$strandString";
		
			print OUT $outHead;
			print OUT "$back";
		}	
		
		close OUT;
		
		# produce alignment			
		$cmd = "";
		my $outFileAli = $outFileFa;
		
		unless ($C{aln_cmd}=~m/t_coffee/) {
			if($opt_out==1) {
				$cmd = "$C{aln_cmd} $outFileFa";
			}
			else {
				$cmd = "$C{aln_cmd} $outFileFa >/dev/null";
			}
		} 
		
		system("$cmd") == 0 or die $die_msg."system call '$cmd' failed: $?";
		if($C{aln_cmd} =~ /dialign/) {
			$outFileAli =~ s/.fa(sta)*$/.ali/;
			$outFileAli =~ /^(.+).ali$/;
			$cmd = "mv $outFileAli $1.aln";
			$outFileAli = "$1.aln";
			system($cmd) ==0 or die $die_msg."Can not move dialign alignment file '$outFileAli' by cmd='$cmd'";
		}
		
		print "Aln created ==> '$outFileAli'\n" if($opt_out==1);
			
		$step++;
	}
	else {
		print "Aln rejected, too less sequences: $qtMembers!\n" if($opt_out==1);
	}
	
	print "\n" if($opt_out==1);

	%seenPos = ();
}


# try to add the subject to the aln
sub addThisSubject {
	my ($query, $pS, $pE, $eV, $sS, $sE) = @_;
	
	my $l = ($pE - $pS + 1);
	
	# only one hit is stored, we directly reject paraloges
	# if there are >1 hits for one organism, 
	# the longest comes into the alignment
	foreach my $subject (sort keys %{$data{$query}{"$pS:$pE"}}) {
		# Tvi_Tviv477b03.p1k:000000000001:12853:+:1720:1796:0:3e-21
		$subject =~ /^(\S\S\S)\_(\d+)(\:\d+\:\d+\:[+|-])\:(\d+)\:(\d+)\:([0|1])\:(.+)$/;
		
		if(! exists ($subs{$1}) ) {
			$subs{$1}{qLength}=0;
		}
		 # is there a better length?
		if($subs{$1}{qLength}<$l) {
			$subs{$1}{qLength}=$l;
			$subs{$1}{qS}=$pS;
			$subs{$1}{qE}=$pE;
			$subs{$1}{sS}=$4;
			$subs{$1}{sE}=$5;
			$subs{$1}{subStrand}=$6;
			$subs{$1}{eV}=$7;
			$subs{$1}{data}=$subject;
		}
		elsif($subs{$1}{qLength}==$l) { # better evalue?
			if($eV<$subs{$1}{eV}) {
				$subs{$1}{qS}=$pS;
				$subs{$1}{qE}=$pE;
				$subs{$1}{sS}=$4;
				$subs{$1}{sE}=$5;
				$subs{$1}{subStrand}=$6;
				$subs{$1}{eV}=$7;
				$subs{$1}{data}=$subject;
			}
		}
		
		die $die_msg."Empty entry: eV='$subs{$1}{eV}' at subject='$subject'!" if($subs{$1}{eV} eq '');
	}
}

# adding flanking region (FR) to a start coordinate
# the length of the FR is defined in the cfg file
sub addFlankingStart {
	my ($start) = @_;
		
	if( $start<=$C{flanking} ) {
		$start=1;
	}
	else { # >
		$start -= $C{flanking}; # add downstream flanking region
	}
	
	return $start;
}

# adding flanking region to an end coordinate
# the length of the FR is defined in the cfg file
sub addFlankingEnd {
	my ($end, $regionLength) = @_;
	
	if( ($end+$C{flanking}-1)>=$regionLength ) {
		$end=$regionLength;
	}
	else { # <=
		$end += $C{flanking}; # add upstream flanking region
	}
	
	return $end;
}


# read in the blast files
sub hashInputNew {
	# hash input data for each file
	foreach my $file(sort {$a cmp $b} @Files) {
		if($file =~ /^(\d+)\..+\.merged$/) { # if file is blast result
						
			open(IN, "$resDir/$file") or die $die_msg."Could not open blast output '$resDir/$file'!";
			my @data = <IN>;
			close(IN);
			
			print "hashing file: $file\n" if($opt_s==0);
						
			# for each line
			for(my $i=0; $i<=$#data; $i++) {
				if($data[$i] !~ /^$/) {					
					%line = %{readInputLine($data[$i])};	

					# we have to adjust the positions of the query/subject to make them sortable
					# 123 -> 000123 depending on maximum genome size
								
					# Bacillus_subtilis_subsp_subtilis_str_168:9460:6452:+
					if( $line{query} =~ /^(...)(.+)\:(\d+)\:(\d+):(.)/ ) {
						my $org = $1;
											
						# consistency check to get more robust
						# ensures that query and subject are specified at the cfg file
						# there are people who change the cfg file
						# (remove organisms)
						# but do not remove the obsolete blast hits
						if(exists $ORG{$org}) {
							$line{query} = "$org$2:".(addZeros($3)).":$4:$5";
						}
						else {
							die $die_msg."Invalid organism (query): '$org'!";
						}							
					}
					else {
						die $die_msg."Wrong query format: '$line{query}'! Must match ".quotemeta('/^(...)(.+)\:(\d+)\:(\d+):(.)/')."!";
					}
					
					if( $line{subject} =~ /^(...)(.+)\:(\d+)\:(\d+):(.)/ ) {
						my $org = $1;
						
						if(exists $ORG{$org}) {
							$line{subject} = "$org$2:".(addZeros($3)).":$4:$5";
						}
						else {
							die $die_msg."Invalid organism (subject): '$org'!";
						}
					}
					else {
						die $die_msg."Wrong subject format: '$line{subject}'! Must match ".quotemeta('/^(...)(.+)\:(\d+)\:(\d+):(.)/')."!";
					}
					
					$line{qS} = addZeros($line{qS});
					$line{qE} = addZeros($line{qE});
											
					# only use hits that are long enough
					if( ($line{qE}-$line{qS}+1)>=$C{minAlnLen} ) {
						$data{$line{query}}{"$line{qS}:$line{qE}"}{"$line{subject}:$line{sS}:$line{sE}:$line{subStrand}:$line{eV}"}=$data[$i];
					}					
				}
			} # for each line
		}
	} # for each file
}



sub addZeros {
	my ($string) = @_;
		
	my $qtDigits=length($string);
	my $diff = $maxDigitSize-$qtDigits;

	my $zeros='';
	
	for(my $k=1; $k<=$diff; $k++) {
		$zeros .= "0";
	}						
	
	return "$zeros$string";
}



# Parses lines of bla[s]t input files and returns a hash storing the line data
# The format of the lines of the result files differ: We have to divide between combined=1 and combined=0 files.
sub readInputLine {
	my($line) = @_;	
	
	my %hash;
	
	#combined=1 query=Bacillus_subtilis_subsp_subtilis_str_168:160544:19049:- subject=Bacillus_cereus_ATCC_10987:330810:5595:+ alnLen=1109 qS=11397 qE=12505 sS=1 sE=1112 eV=195e-273 how=11989_12505_595_1112,11397_11983_1_587
	if($line =~ /^combined=(\d)\s+query=(.+)\s+subject=(.+)\s+alnLen=(\d+)\s+qS=(\d+)\s+qE=(\d+)\s+sS=(\d+)\s+sE=(\d+)\s+eV=(.+)\s+how=(.+)$/) {
		
		$hash{combined}	= $1;
		$hash{query}	= $2;
		$hash{subject}	= $3;
		$hash{alnLen}	= $4;
		$hash{qS}	= $5;
		$hash{qE}	= $6;
		$hash{qLen}	= $6-$5+1;				
		$hash{sS}	= $7;
		$hash{sE}	= $8;
		$hash{subStrand}= 1;
		
		# swap positions if subject lies on negative strand, so that start is < than end
		if($hash{sS} > $hash{sE}) {
			$hash{sS}=$8;
			$hash{sE}=$7;
			$hash{subStrand} = 0;
		}
		
                $hash{eV}	= $9;
		$hash{how}	= $10;		
	}
	# combined=0 query=Bacillus_subtilis.gbk:1072418:4326:+ subject=Bacillus_cereus:1:5224283:+ percIdent=69.83 alnLen=232 qtMM=70 qtGO=0 qS=3540 qE=3771 sS=1162647 sE=1162878 eV=8.8e-54 bitS=208.0 how=3540_3771_1162647_1162878
	elsif($line =~ /^combined=(\d)\s+query=(.+)\s+subject=(.+)\s+percIdent=(.+)\s+alnLen=(\d+)\s+qtMM=(\d+)\s+qtGO=(\d+)\s+qS=(\d+)\s+qE=(\d+)\s+sS=(\d+)\s+sE=(\d+)\s+eV=(.+)\s+bitS=(.+)\s+how=(.+)$/) {
		
		$hash{combined}	= $1;
		$hash{query}	= $2;
		$hash{subject}	= $3;
		$hash{percIdent}= $4;
		$hash{alnLen}	= $5;
		$hash{qtMM}	= $6;
		$hash{qtGO}	= $7;
		$hash{qS}	= $8;
		$hash{qE}	= $9;
		$hash{qLen}	= $9-$8+1;
		$hash{sS}	= $10;
		$hash{sE}	= $11;
		$hash{subStrand}= 1;
		
		# swap positions if subject lies on negative strand, so that start is < than end
		if($hash{sS} > $hash{sE}) {
			$hash{sS}=$11;
			$hash{sE}=$10;
			$hash{subStrand} = 0;
		}
		
		$hash{eV}=$12;
		$hash{bitS}=$13;
		$hash{how}=$14;		
	}
	else {
		die $die_msg."Unknown line format:\nline='$line'";
	}
	
	# Checking consistency
	die $die_msg."qS=$hash{qS} > qE=q$hash{qE}! Not allowed! (line=$line)"if($hash{qS}>$hash{qE});
	die $die_msg."sS=$hash{sS} > sE=q$hash{sE}! Not allowed! (line=$line)"if($hash{sS}>$hash{sE});
	die $die_msg."Empty value: qS='$hash{qS}', qE='$hash{qE}', eV='$hash{eV}'" if($hash{qS} eq '' || $hash{qE} eq '' || $hash{eV} eq '' || $hash{sS} eq '' || $hash{sE} eq '');
	# now start is always < than end
	
	return \%hash;
} # readInputLine()


# remove 0s from strings like
# Bacillus_subtilis_subsp_subtilis_str_168:0026337:474:+
sub removeNulls {
	my ($string) = @_;
	
	$string =~ /^(.+)\:(\d+)\:(\d+):([+|-])/;
	my $tmp1 = $1;
	my $tmp2 = $2;
	my $tmp3 = $3;
	my $tmp4 = $4;
	if($tmp2 =~ /^0/) {
		$tmp2 =~ s/^0+//g;
	}
	my $ret = $tmp1.":".$tmp2.":".$tmp3.":".$tmp4;
	
	return $tmp1.":".$tmp2.":".$tmp3.":".$tmp4;
}

__END__

=head1 NAME

C<realign.pl> - part of the NcDNAlign alignment pipeline, step (4)

Realignment of prior BLAST hits.

=head1 SYNOPSIS

realign.pl [options]

=head1 OPTIONS

=over 8

=item B<-c, --conf> FILE

Path to central NcDNAlign configuration file [REQUIRED]

=item B<-o, --out> [0|1]

Printing detailed results at STDOUT ON(1) or OFF(0). Default: 0.

=item B<-s, --silent> [0|1]

Silent mode, avoid printing to STDOUT
0=OFF (Default), 1=ON

=item B<-v, --version>

Prints version information and exits.

=item B<-h, --help>

Prints a short help message and exits.

=item B<--man>

Prints a detailed manual page and exits.

=back

=head1 DESCRIPTION

Traditionally, MSAs are set up by pairwise alignments. We follow this strategy by grouping corresponding BLAST HSPs to one single MSA. Therefore, all HSPs are sorted according to the loci of the reference genome. Afterwards, one 'best' subject sequence is selected for each reference locus that overlaps the reference. It is ensured that every new subject sequence overlaps with all previous ones of the hitherto MSA. FThe minimal length for acceptance of an alignment is defineable. We recommend a value of 40 nt for ncRNA prediction and gene-finding on a genome-wide scale. Optionally, alignable subsequences can be enlarged by a certain user-defined flanking region. This compensates putative BLAST alignment errors, whenever initial seed alignments are not sufficiently extended to HSPs. Finally, MSAs of all accepted regions are computed using DIALIGN. Furthermore, the script creates a BED file 'gwAln.bed' keeping the coordinates of the alignment input respectively initial BLAST hits.

Be careful: A temporary file 't.NcDNAlign' must be createable in the screens' directory!

=head1 EXAMPLES

$ realign.pl -c config-file.cfg

=head1 AUTHORS

Dominic Rose (dominic@bioinf.uni-leipzig.de)

=head1 AVAILABILITY

http://www.bioinf.uni-leipzig.de/Software/NcDNAlign/

=cut


