#!/usr/bin/perl -w

########################################################
# 
# trimAln.pl 
#    part of the NcDNAlign alignment pipeline, step (5)
#
# Author: Dominic Rose
#
# Contact: dominic@bioinf.uni-leipzig.de
# http://www.bioinf.uni-leipzig.de/Software/NcDNAlign/
#
########################################################

use NcDNAlign;
use strict;
use File::Find;
use Math::Round;
use Getopt::Long;
use Pod::Usage;

my ($opt_conf, $opt_maxGaps, $opt_maxZeros, $opt_dropRef, $opt_out, $opt_stat, $opt_r, $opt_f, $opt_help, $opt_version, $opt_man, $opt_tree, $opt_s) 
	= ('', 120, 20, 0, 0, 0, 1, "CLUSTAL", '', '', '', 0, 0);

GetOptions(
	"c|conf=s" 	=> \$opt_conf,
	"g|maxGaps:i"	=> \$opt_maxGaps,
	"z|maxZeros:i"	=> \$opt_maxZeros,
	"d|dropRef:i"	=> \$opt_dropRef,
	"stat:i" 	=> \$opt_stat,
	"r|realign:i" 	=> \$opt_r,
	"t|tree:i"	=> \$opt_tree,
	"o|out:i" 	=> \$opt_out,
	"f|format:s"	=> \$opt_f,
	"h|help"	=> \$opt_help,
	"v|version"	=> \$opt_version,
	"man"		=> \$opt_man,
	"s|silent:i"	=> \$opt_s
) or pod2usage(2);

pod2usage(1) if ($opt_help || $opt_s !~ /^[1|0]$/);
pod2usage(-exitstatus => 0, -verbose => 2) if $opt_man;

if($opt_version) {
	print "This is cutSequences.pl, part of the NcDNAlign package, version: $VERSION.\n$url\n";
	exit;
}

pod2usage(1) if(!$opt_conf);
pod2usage(1) if($opt_out !~ /^[0|1]$/ || $opt_stat !~ /^[0|1]$/ || $opt_r !~ /^[0|1]$/ || $opt_dropRef !~ /^[0|1]$/);

if($opt_f !~ /CLUSTAL|FASTA|MAF/) {
	print STDERR "ERROR: Wrong alignment format: '$opt_f'!";
	pod2usage(1);
}


# ------ Start ----------
loadConfig($opt_conf);


my %seq;

my $outFileTrimmedPos;
my $line_length 	= 60; # line length of clustalw
my $badAlnDir 		= "$C{p_dir}/bad";
my ($newTrimAlnLen, $oldTrimAlnLen);
my $outFile;
my $outFileDND;
my $alignLength;
my $qtAll;
my $qtAlignments 	= 0;
my $tmpOutFileFa 	= "$C{p_dir}/t.fa";
my $tmpOutFileDND	= "$C{p_dir}/t.dnd";
my $fileNumber 		= 0;
my $floatFileNr 	= 0;
my $trimmedLength 	= 0;
my ($maxLeftGapLenID, $maxLeftGapLen, $maxRightGapLenID, $maxRightGapLen) = (0, 0, 0, 0);
my %looser;

# DIALIGN specific global variables
my $digitString = '';
my %pos;

my $bedFile = "$C{p_dir}/trimmedAln.bed"; #
my %bed;

# Statistics
my %stat;
$stat{sumSeqsPerAln} = 0;

main();

sub main
{
	$qtAll = keys %ORG;
	$outFileTrimmedPos = $C{aln_dir}."/trimmedPos.dat";
	system("rm $outFileTrimmedPos") if (-f $outFileTrimmedPos);
	open(TRIMOUT, ">$outFileTrimmedPos") or die $die_msg."Cant open outfile='$outFileTrimmedPos'!";
		
	# for each aln file
	find(\&wanted, $C{aln_dir});
	
	close(TRIMOUT);
	
	# statistics 
	if($opt_stat==1) {
		if($qtAlignments >0 ) {
					
			my $string = "\n\nSTATISTICS:\n-----------\n";
			$string .= "# Alignments  = $qtAlignments\n";
			$string .= "mean Seqs/Aln = ".nearest(0.01, $stat{sumSeqsPerAln}/$qtAlignments)."\n";
			
			if($opt_out==1) {
				print STDOUT $string;
			}
			
			my $stat_file = "$C{p_dir}/aln.statistics";
			
			open (OUT ,"> $stat_file") or die $die_msg."Cant open outfile='$stat_file'!";
			print OUT $string;
			close(OUT);
			print "Alignment statistics                    => '$stat_file'\n" if($opt_s==0);	
		}
	}
	
	open(BED, ">$bedFile") or die $die_msg."Could not open file '$bedFile'!";
	# write bed file
	foreach my $organism (sort keys %bed) {
		foreach my $chromosome (sort {$a<=>$b} keys %{$bed{$organism}}) {
			foreach my $start (sort {$a<=>$b} keys %{$bed{$organism}{$chromosome}}) {
				my @e = split /;/, $bed{$organism}{$chromosome}{$start};
				die $die_msg."Exception: Wrong bed format!" if(scalar(@e) != 5);
				print BED "$e[4]\t$e[0]\t$e[1]\t$e[2]\t.\t$e[3]\n";
			}
		}
	}
	close BED;
	print "BED file of trimmed alignment positions => '$bedFile'\n\n" if($opt_s==0);
	
}# end of main


# perform actions on every aln file
sub wanted {
	my $file = $File::Find::name;
			
	if($file =~ /(\d+)\.aln$/ ) {	
		$fileNumber = $1;
				
		if($opt_out==1) {
			print "------ $_ ------\n";
		}
		else {
			print "$_\n" if($opt_s==0);
		}
		
		$alignLength = read_sequences($file); 	# set up %seq hash
		
		parseDigitString();			# get valid positions
		getAln($file);				# get the alignments and sequences due to the %pos hash	
	}		
} # end of wanted


# Trim leading and preceeding gaps of multiple alignment.
# Trimmed alignment starts with the first contiguous block of aligned letters in all sequences and 
# ends with such a block! Regard N's or n's also as leading or preceeding gaps.
sub getMinOverlap {
	my @Keys = getSortedKeys();
			
	## FIND POSITIONS WHICH WILL BE TRIMMED
	# Sequences have same length as they start and end with matching regions!
	my $length = length $seq{$Keys[0]}{splitSeq};
	my $startTrim = 0;

	for(my $i=0; $i<$length; $i++){
		my $column = get_column($i, %seq);
		# As long column contains a gap continue
		if($column =~ /.*[\-Nn]+.*/){ next; }
		else { # First match in all sequences found
			$startTrim = $i;
			last;
		}
	}
	
	# Same from the end of the alignment
	my $endTrim = $length;
	for(my $i=$length-1; $i>=0; $i--) {
		my $column = get_column($i, %seq);
		# As long column contains a gap continue
		if($column =~ /.*[\-Nn]+.*/){ next; }
		else { # First match in all sequences found
			$endTrim = $i;
			last;
		}
	}
	
	# get trimmed Pos
	foreach my $k(@Keys) {
		# Here, do not reset N's or n's as they are actually part of the sequence!
		my $seqStart = substr $seq{$k}{splitSeq}, 0, $startTrim;
		$seqStart =~ s/-//g;
		my $seqEnd = substr $seq{$k}{splitSeq}, $endTrim;
		$seqEnd =~ s/-//g;
		print "seqStart='$seqStart', seqEnd='$seqEnd', startTrim='$startTrim', endTrim='$endTrim'\n" if($opt_out==1);
		$seq{$k}{lTrimmedNucleoLen} =  (length $seqStart) + $seq{$k}{overallLeftLostNt};
		$seq{$k}{rTrimmedNucleoLen} = (length $seqEnd) -1 + $seq{$k}{overallRightLostNt};
		print 	"getMinOverlap: seq{$k}{lTrimmedNucleoLen}='$seq{$k}{lTrimmedNucleoLen}', ".
			"seq{$k}{rTrimmedNucleoLen}='$seq{$k}{rTrimmedNucleoLen}' overallLeft='$seq{$k}{overallLeftLostNt}', overallRight='$seq{$k}{overallRightLostNt}'\n"  if($opt_out==1);
	}	

	## TRIM SEQUENCES
	foreach my $k(@Keys) {
		$seq{$k}{trimmedSeq} = substr $seq{$k}{splitSeq}, $startTrim, $endTrim-$startTrim+1;
		$seq{$k}{trimmedSeqLen} = length $seq{$k}{trimmedSeq};		
		$seq{$k}{lGapsLen} = 0; # just init
		$seq{$k}{rGapsLen} = 0;
		if( $seq{$k}{splitSeq} =~ /^(\-+)([a-zA-Z].*[a-zA-Z])(\-+)$/ ) {
			$seq{$k}{lGapsLen} = length $1;
			$seq{$k}{rGapsLen} = length $3;
		}
		$seq{$k}{lTrimLen} = 0; # init
		$seq{$k}{rTrimLen} = 0;

		# count all gaps of the trimmed sequence
		$seq{$k}{qtGaps} = $seq{$k}{trimmedSeq}=~ tr/\-//;
	}
	
	my ($lTrimLenMax, $rTrimLenMax) = (0, 0);
	
	# The TrimLen is the maximum of the GapLen
	foreach my $k(@Keys) {
		if( $seq{$k}{lGapsLen} > $lTrimLenMax) {
			$lTrimLenMax = $seq{$k}{lGapsLen};
		}
			
		if( $seq{$k}{rGapsLen} > $rTrimLenMax ) {
			$rTrimLenMax = $seq{$k}{rGapsLen};
		}		
	}
		
	# set it
	foreach my $k(@Keys) {	
		$seq{$k}{lTrimLen} = $lTrimLenMax;
		$seq{$k}{rTrimLen} = $rTrimLenMax;
	}
	
	$trimmedLength = $seq{$Keys[0]}{trimmedSeqLen};

	# Now write trimmed sequences into outfile
	open(OUT, ">$outFile") or die "Could not create file $outFile!";
	print OUT "CLUSTAL W\n\n";
	
	my $row=0;
	 
	for(my $i=0; $i<$trimmedLength; $i += $line_length) {
		my %starSearch;
		$row++;		
		#print "i=$i row=$row\n";
		
		foreach my $k(@Keys) {
			#print "k=$k\n";
			if($seq{$k}{trimmedSeq} eq '') {
				die $die_msg."Could not find sequence '$seq{$k}{trimmedSeq}'!";
			}
			
			printf OUT "%-40s", $k;
			printf OUT "%s\n", substr($seq{$k}{trimmedSeq}, $i, $line_length);
			$starSearch{$k}{splitSeq} = $seq{$k}{trimmedSeq}; # for the get_column function
		}
		
		# GIVE ME THE STARS 
		my $empty='';
	
		for(my $j=0; $j<32; $j++) { ### ??? why not 40 ???
			$empty .= ' ';
		}
		
		print OUT "$empty\t";
		
		for(my $j=0; $j<$line_length; $j++) {
			my $whichColumn = $j+(($row-1) * $line_length);
			my $column = get_column($whichColumn, %starSearch);
			
			last if($column eq '');
			
			my $firstChar = substr $column, 0, 1;
			if( $column =~ /^$firstChar+$/ ) {
				print OUT "*";
				#print "*";
			}
			else {
				print OUT " ";
			}
		}
		print OUT "\n\n";
	}

	close(OUT);

	if( $trimmedLength == 0 ) {
		die .$die_msg."No minimal overlap!";
	}
		
	return $trimmedLength;

}# end of trim_alignment





sub beautification {
	my ($weHaveTooMuchGaps, $badSeq) = @_;
	
	if($weHaveTooMuchGaps eq 1 ) {
		print "\t calculating getMinOverlap because of weHaveTooMuchGaps='$weHaveTooMuchGaps'\n" if($opt_out==1);
		$trimmedLength = getMinOverlap($outFile);
	}
	
	my $goOn=1;
	my $round=0;
	($newTrimAlnLen, $oldTrimAlnLen) = ($trimmedLength, 0);

	# apply beautification
	while($goOn==1 && ($trimmedLength<$C{minAlnLen} || $weHaveTooMuchGaps==1) ) {
		
		my ($weHaveLGaps, $weHaveRGaps) = (0, 0);
		my ($qtLeft, $qtRight) = (1, 1); # themselves
		($maxLeftGapLenID, $maxLeftGapLen, $maxRightGapLenID, $maxRightGapLen) = 
			(0, 0, 0, 0);
		my ($minlNucleoLen, $minlNucleoLenID, $minrNucleoLen, $minrNucleoLenID) = 
			($alignLength-$trimmedLength+1, 0, $alignLength-$trimmedLength+1, 0);
		my ($qtHasLeftNucleotids, $qtHasRightNucleotids, $lNucleoLen, $rNucleoLen) = 
			(0, 0, 0, 0, 0, 0);
		my $qtOrg = 0; # how many valid organisms are in this alignment
		
		$round++;
		print "\nROUND: $round, alignLength=$alignLength vs trimmedLength=$trimmedLength!\n" if($opt_out==1);
				
		my @keys = getSortedKeys();
		
		foreach my $k (@keys) {
			$qtOrg++;
						
			# are there gaps at the borders?
			if($seq{$k}{lGapsLen}>0) {
				$weHaveLGaps=1;
				print "There are '$seq{$k}{lGapsLen}' gaps at the left of '$k'!\n" if($opt_out==1);
			}
			
			if($seq{$k}{rGapsLen}>0) {
				$weHaveRGaps=1;
				print "There are '$seq{$k}{rGapsLen}' gaps at the right of '$k'!\n" if($opt_out==1);
			}				
		}

		if( $newTrimAlnLen == $oldTrimAlnLen ) {
			print "Can't get better ... trimming stopped!\n" if($opt_out==1);
			$goOn=0;
			last;
		}
								
		# Are there enough sequences in the alignment to go on? 
		# we need one more org than the minimum number 
		# because one will be dropped to improve the alignment
		if( $qtOrg > $C{min_seq} ) {
			if($opt_out==1) {
				print "Beautification starts!\n"; 
	
				print "Splitted:\n";								
				foreach my $k (@keys) {
					print "$k   $seq{$k}{splitSeq}\n";
				}
			
				print "Trimmed:\n";				
				foreach my $k (@keys) {
					print "$k   $seq{$k}{trimmedSeq}\n";
				}
			}
									
			foreach my $k (@keys) {
								
				$lNucleoLen = $seq{$k}{lTrimLen} - $seq{$k}{lGapsLen};
				$rNucleoLen = $seq{$k}{rTrimLen} - $seq{$k}{rGapsLen};

				if($opt_out==1) {
					print "\t $k:  lNucleoLen=$lNucleoLen = lTrimLen=$seq{$k}{lTrimLen} - lGapsLen=$seq{$k}{lGapsLen}\n";
					print "\t $k:  rNucleoLen=$rNucleoLen = rTrimLen=$seq{$k}{rTrimLen} - rGapsLen=$seq{$k}{rGapsLen}\n";
				}
				$k =~ /^\d+\.(.+)$/;				
							
				# should we have a look at the left or at the right of the initial minimal overlap?
				# left:
				# have a look at the sequence that has the minimal $lNucleoLen, this is the border sequence
				# how many sequences are there having lGapsLen <= $seq{border}{lGapsLen}
				# they have to be counted
				# right: similar
				if($opt_dropRef==1) { # no test, it is allowed to drop target
					getMaxGapLen($k);
				}
				else { # dropRef == 0
					if($k !~ /$C{target_name}/) { # never drop sequence of target organism				
						getMaxGapLen($k);
					}
				}

				# left or right?
				if($seq{$k}{lGapsLen} >=0) {
											
					$qtHasLeftNucleotids++ if($lNucleoLen>0);							
				}
					
				if($seq{$k}{rGapsLen} >=0) {
											
					$qtHasRightNucleotids++ if($rNucleoLen>0);							
				}
				
				# The target sequence can also be the border
				if( $lNucleoLen>0 && $lNucleoLen <= $minlNucleoLen ) {
					# new l min
					
					$minlNucleoLen	 = $lNucleoLen;
					$minlNucleoLenID = $k;
				}
										
				if( $rNucleoLen>0 && $rNucleoLen <= $minrNucleoLen ) {
					# new r min
					
					$minrNucleoLen	 = $rNucleoLen;
					$minrNucleoLenID = $k;
				}
			}
			
			# left or right???
			$qtLeft = 0;
			$qtRight= 0;
			
			foreach my $k (@keys) {
					
				if( $minlNucleoLenID ne 0 && $weHaveLGaps==1  && $minlNucleoLen>0 && $seq{$k}{lGapsLen} <= $seq{$minlNucleoLenID}{lGapsLen}) {
					$qtLeft++;
				}
			
				if( $minrNucleoLenID ne 0 && $weHaveRGaps==1 && $minrNucleoLen >0 && $seq{$k}{rGapsLen} <= $seq{$minrNucleoLenID}{rGapsLen}) {
					$qtRight++;
				}
			}
			
			# we are done if there are no nucleotides we could use to enlarge the alignment
			if($qtLeft==0 && $qtRight==0 || ($minlNucleoLenID eq 0 && $minrNucleoLenID eq 0)) { 
				print "\t File='$outFile' is trimmed correct! (qtLeft=$qtLeft==0 && qtRight=$qtRight==0 || ($minlNucleoLenID==0 && $minrNucleoLenID==0))\n\t Go on with next alignment!\n" if($opt_out==1);
				$goOn=0;
			}
			else {
				if($opt_out==1) {
					print "\t Left or right? ". 
					"(minlNucleoLen=$minlNucleoLen by $minlNucleoLenID, ".
					"minrNucleoLen=$minrNucleoLen by $minrNucleoLenID)\n" ;
			
					print "\t looking left : border set by $minlNucleoLenID, affecting $qtLeft sequences\n";
					print "\t looking right: border set by $minrNucleoLenID, affecting $qtRight sequences\n";
				}
				
				if( 	$qtLeft>1 && (	# you can only get better if there are more than one candidates on the line of the border
					($minrNucleoLenID eq 0 && $minlNucleoLenID ne 0) || # go to the left, if there is nothing at the right but at the left
					($qtLeft>$qtRight))
				) {	
					print "\t DECISION: LEFT (qtLeft=$qtLeft > qtRight=$qtRight, ".
					"minrNucleoLenID='$minrNucleoLenID', ".
					"minlNucleoLenID='$minlNucleoLenID', $qtLeft wins):\n" if($opt_out==1);

					trim_withLeftBorder($qtLeft, $qtRight, $maxLeftGapLenID); # trim it!
				}
				elsif( $qtRight>1 && (	# you can only get better if there are more than one candidates on the line of the border
							($minlNucleoLenID eq 0 && $minrNucleoLenID ne 0) || # go to the right, if there is nothing at the left but at the right
							($qtLeft<$qtRight)
						)
				) {
					print "\t DECISION: RIGHT (qtLeft=$qtLeft < qtRight=$qtRight, $qtRight wins)\n" if($opt_out==1);	
					trim_withRightBorder($qtLeft, $qtRight, $maxRightGapLenID); # trim ist
				}
				else { 
					if($qtLeft<=1 && $qtRight<=1) {
						print "\t No more trimmable sequences! Alignment is trimmed correct!\n" if($opt_s==0);;
						$goOn=0;
						last;
					}
					
					if($opt_out==1)	{
						print "\t qtLeft=qtRight  ->  lets have a closer look! (weHaveLGaps=$weHaveLGaps, weHaveRightGaps=$weHaveRGaps)\n";
						print "\t left candidate:  '$maxLeftGapLenID'\n";
						print "\t right candidate: '$maxRightGapLenID'\n";
					}
					
					if($maxLeftGapLenID eq $maxRightGapLenID ) {
						print "\t DECISION: LEFT (qtLeft=$qtLeft == qtRight=$qtRight, ".
						"minrNucleoLenID='$minrNucleoLenID', ".
						"minlNucleoLenID='$minlNucleoLenID', $qtLeft wins):\n" if($opt_out==1);

						trim_withLeftBorder($qtLeft, $qtRight, $maxLeftGapLenID); # trim it, doesnt matter if left or right!
					}
					else {
						if($opt_out==1) {
							print "\t minlNucleoLen=$minlNucleoLen\n";
							print "\t minrNucleoLen=$minrNucleoLen\n";
						}
						
						if($minlNucleoLen>=$minrNucleoLen) {
							print "\t DECISION: LEFT (because $minlNucleoLen>=$minrNucleoLen)\n" if($opt_out==1);
							trim_withLeftBorder($qtLeft, $qtRight, $maxLeftGapLenID); # trim it
						}
						else {
							print "\t DECISION: RIGHT (because $minlNucleoLen<$minrNucleoLen)\n" if($opt_out==1);
							trim_withRightBorder($qtLeft, $qtRight, $maxRightGapLenID); # trim ist	
						}
					}			
				}
			}
			
			# test gaps for next round
			($weHaveTooMuchGaps, $badSeq) = testForTooMuchGaps();
		}
		else {
			print "Not enough sequences for further beautification in the alignment. qtOrg=$qtOrg!\n" if($opt_out==1);
			
			# to few sequences for beautification
			# but same time too much gaps in one sequence of the alignment
			# The alignment may get lost due to the gap-score
			# The gap-score could throw out one sequence, then we will losse the whole alignment.
			# solution: split the alignment to ensure that not everything is rejected.
			if($weHaveTooMuchGaps==1) {
				splitAlignment($badSeq);				
			}

			$goOn=0;
			last;
		}
	}
}

# split the alignment if one sequence contains too much gaps and 
# alignment may be lost because of too few sequences (gap-score may throw out the last necessary sequence)
sub splitAlignment {
	my ($badSeq) = @_;
	
	# get the position of the first occurence of the gap desert
	my $gapSeq = "";
	for(my $i=1; $i<=$opt_maxGaps; $i++) {
		$gapSeq .= "-";
	}
	
	my $pos = index $seq{$badSeq}{trimmedSeq}, $gapSeq;
	
	# check whether there are left or right more nucleotides;
	# the side with more nucleotides will be the winning side
	# we keep this alignment fragment
	my ($qtLeftNuc, $qtRightNuc, $leftSub, $rightSub) = (0, 0);
	
	$leftSub = substr $seq{$badSeq}{trimmedSeq}, 1, $pos-1;
	$rightSub = substr $seq{$badSeq}{trimmedSeq}, $pos;
	
	$qtLeftNuc = $leftSub =~ tr/\-//;
	$qtRightNuc = $rightSub =~ tr/\-//;
	$qtLeftNuc = length($leftSub) - $qtLeftNuc;
	$qtRightNuc = length($rightSub) - $qtRightNuc;

	if($opt_out==1) {
		print "Bad sequence: '$badSeq'!\n";
		print "Splitting at '$pos':\n";
		print "qtLeftNuc='$qtLeftNuc' vs qtRightNuc='$qtRightNuc'\n";
	}


	if($qtLeftNuc>$qtRightNuc) {
		if($opt_out==1) {
			print "keeping left fragment:\n";
		}
		
		setupFragment(1,$pos-1);
	}
	elsif($qtLeftNuc<$qtRightNuc) {
		if($opt_out==1) {
			print "keeping right fragment:\n";
		}
		
		setupFragment($pos-1, length($seq{$badSeq}{trimmedSeq}));
	}
	else { # ==
		if($opt_out==1) {
			print "keeping left fragment:\n";
		}
		
		setupFragment(1,$pos-1);
	}	
}

# updating seq hash to fragment
sub setupFragment {
	my ($startTrim, $endTrim) = @_;
	
	my @Keys = getSortedKeys();

	#####
	foreach my $s (@Keys) {
		
		$seq{$s}{splitSeq} 	= substr $seq{$s}{originalSeq}, $startTrim, $endTrim-$startTrim+1;
		$seq{$s}{splitSeqLen}	= length $seq{$s}{splitSeq};
		$seq{$s}{trimmedSeq}	= $seq{$s}{splitSeq};
		$seq{$s}{trimmedSeqLen}	= $seq{$s}{splitSeqLen};
		
		my ($prefix, $postfix) = ('', '');
		$prefix  = substr $seq{$s}{originalSeq}, 0, $startTrim;
		$postfix = substr $seq{$s}{originalSeq}, $endTrim;

		$prefix =~ s/-//g;
		$postfix =~ s/-//g;;
		
		$seq{$s}{lTrimmedNucleoLen} 	= length $prefix;
		$seq{$s}{rTrimmedNucleoLen} 	= (length $postfix)-1; # -1 bug compensation for end of trimmedAln.bed
		$seq{$s}{overallLeftLostNt}	= $seq{$s}{lTrimmedNucleoLen};
		$seq{$s}{overallRightLostNt}	= $seq{$s}{rTrimmedNucleoLen};
	}
	
	
	$trimmedLength = $seq{$Keys[0]}{trimmedSeqLen};
	#####
	
	
	if($opt_out==1) {
		print "Kept fragment:\n";
				
		foreach my $k (@Keys) {
			print "$k   $seq{$k}{trimmedSeq}\n";
		}
	}
}



sub testForTooMuchGaps {
	my $weHaveTooMuchGaps = 0;
	my @keys = getSortedKeys();
	
	my $badSeq = "";
	
	foreach my $s (@keys) {
		if($seq{$s}{trimmedSeq} =~ /-{$opt_maxGaps,}/) {
			print "Recognised >= $opt_maxGaps gaps at '$outFile' by '$s'\n" if($opt_out==1);
			$weHaveTooMuchGaps=1;
			
			$badSeq = $s;
			
			last;
		}
	}
	
	return $weHaveTooMuchGaps, $badSeq;
}

# get the alignments due to the %pos hash
sub getAln {
	my ($file) = @_;
	
	my ($trimmedPosStart, $trimmedPosEnd) = (0,0);

	if($opt_out==1) {
		print "before:\n";
		#foreach my $k (sort {$a <=> $b} keys %seq) {
		foreach my $k (sort keys %seq) {
			print "$k\t$seq{$k}{originalSeq}\n";
		}
		
		print "after:\n"
	}
		
	# what did we trim automatically?
	my $tmp = '';
	if ($digitString =~ /^(0+)/ ) {
		$tmp = $1;
	}
	$trimmedPosStart = (length $tmp); 
	$tmp = '';
	if( $digitString =~ /(0+)$/ ) {
		$tmp = $1;
	}
	$trimmedPosEnd = (length $tmp);	
	
	my $i=0;
	
	%looser = ();
	
	#foreach my $k (sort {$a <=> $b} keys %pos) {
	foreach my $k (sort keys %pos) {
		
		# restore invalidated hashes of prior alignment fragments
		my @looserkeys = keys %looser;
	
		if( scalar(@looserkeys) >0 ) {
			foreach my $looserkey (@looserkeys) {
				
				foreach my $key (keys %{$looser{$looserkey}} ) {
					# reactivating
					$seq{$looserkey}{$key} = $looser{$looserkey}{$key};
				}				
			}
			%looser = (); # reset for next round
		}
		
		print " ##  pos=$k: ## \n" if ($opt_out==1);
		$i++;
		
		if( $file =~ /(.*\/)(\d+)\.aln/ ) {
			$outFile = "$1$2.$i.aln.trim";
			$outFileDND = "$1$2.$i.dnd";
			$floatFileNr = "$2.$i";
		}
		else {
			die $die_msg."Cant parse file='$file'!";
		}
		
		my @e = split (/\_/, $k);
		die $die_msg."Invalid position: '$k'!" if(scalar(@e) != 2);
		
		#my @keys = sort {$a <=> $b} keys %seq;
		my @keys = sort keys %seq;
		$trimmedLength = $e[1]-$e[0]+1;
		print "setting trimmedLength='$trimmedLength'\n" if($opt_out==1);
		foreach my $s (@keys) {

			my $startTrim	= $e[0]-1;
			my $endTrim 	= $e[0] + $trimmedLength - 1;
			 
			$seq{$s}{splitSeq} 	= substr $seq{$s}{originalSeq}, $startTrim, $endTrim-$startTrim;
			$seq{$s}{splitSeqLen}	= length $seq{$s}{splitSeq};
			$seq{$s}{trimmedSeq}	= $seq{$s}{splitSeq};
			$seq{$s}{trimmedSeqLen}	= $seq{$s}{splitSeqLen};
			
			my ($prefix, $postfix) = ('', '');
			$prefix  = substr $seq{$s}{originalSeq}, 0, $startTrim;
			$postfix = substr $seq{$s}{originalSeq}, $endTrim;

			$prefix =~ s/-//g;
			$postfix =~ s/-//g;;
					
			$seq{$s}{lTrimmedNucleoLen} 	= length $prefix;
			$seq{$s}{rTrimmedNucleoLen} 	= length $postfix;
			$seq{$s}{overallLeftLostNt}	= $seq{$s}{lTrimmedNucleoLen};
			$seq{$s}{overallRightLostNt}	= $seq{$s}{rTrimmedNucleoLen};			
			
			# all keys must be initialized
			$seq{$s}{lGapsLen} = 0; # just init
			$seq{$s}{rGapsLen} = 0;
			if( $seq{$s}{splitSeq} =~ /^(\-+)([a-zA-Z].*[a-zA-Z])(\-+)$/ ) {
				$seq{$s}{lGapsLen} = length $1;
				$seq{$s}{rGapsLen} = length $3;
			}
			
			if($opt_out==1) {
				print "seq{$s}{lTrimmedNucleoLen}='$seq{$s}{lTrimmedNucleoLen}', seq{$s}{rTrimmedNucleoLen}='$seq{$s}{rTrimmedNucleoLen}'\n";
				print "seq{$s}{overallLeftLostNt}='$seq{$s}{overallLeftLostNt}', seq{$s}{overallRightLostNt}='$seq{$s}{overallRightLostNt}'\n";
				print "$s\t$seq{$s}{splitSeq}\n";
				print "\t (lTrim, rTrim) = ($seq{$s}{lTrimmedNucleoLen}, $seq{$s}{rTrimmedNucleoLen})\n";
			}
		}
		
		my ($lTrimLenMax, $rTrimLenMax) = (0, 0);
		
		# The TrimLen is the maximum of the GapLen
		foreach my $k(@keys) {
			if( $seq{$k}{lGapsLen} > $lTrimLenMax) {
				$lTrimLenMax = $seq{$k}{lGapsLen};
			}
				
			if( $seq{$k}{rGapsLen} > $rTrimLenMax ) {
				$rTrimLenMax = $seq{$k}{rGapsLen};
			}		
		}
			
		# set it
		foreach my $k(@keys) {	
			$seq{$k}{lTrimLen} = $lTrimLenMax;
			$seq{$k}{rTrimLen} = $rTrimLenMax;
		}
		
		# trim the flanking regions and get the minimal overlap
		# you can do this everytime
		# but we do it now for only deleting 120 nt gaps
		# 120 is, for example, an appropriate default RNAz window size
		# only do it once
		my ($doWeHaveTooMuchGaps, $badSeq) = testForTooMuchGaps();
		beautification($doWeHaveTooMuchGaps, $badSeq);
		
		# Still too short, we can't apply beautification to enlarge it
		if($trimmedLength<$C{minAlnLen}) { 
			print "Alignment '$outFile' is too short! (trimmedLength=$trimmedLength)\n" if($opt_s==0);
			rejectAln($outFile);
		}
		else {	# long enough, write it out
			
			# get rid of gap enriched sequences!
			my $whatToDo = gap_score();
			if( $whatToDo == 1 ) {
				# Maybe another realignment step in future versions
			}			
			
			# gap_score could have changed alignment composition
			# maybe some sequences got invalidated
			# we must check if the alignment is valid 
			
			# alignment has to be rejected because of worst case in gap_score calculation
			
			#print "File='$outFile', gap-return='$whatToDo'\n";
			
			if($whatToDo == -1) {
				print "Complete alignment is rejected, because the reference consists of gaps only but dropping the reference is not allowed!\n" if($opt_out==1);
				rejectAln($outFile);
			}
			else {
				my $qtS = keys %seq;

				if( $qtS >= $C{min_seq} ) {
					if( $opt_r==1 ) { # realignment 
						performClustalw();
					}
					else { # custom output
						$qtAlignments++;
						writeTrimFile(); # after all
						
						open (OUT, ">$outFile") or die $die_msg."Can not open outfile '$outFile'!";
						
						if($opt_f =~ /CLUSTAL/) {
							writeClustal();
						}
						elsif($opt_f =~ /FASTA/) {
							writeFasta();
						}
						elsif($opt_f =~ /MAF/) {
							writeMAF();
						}
						
						close OUT;
						
						print "outfile ==> '$outFile' is ready! (qtSeq='$qtS')\n\n" if($opt_s==0);
					
						if($opt_stat==1) {
							$stat{sumSeqsPerAln} += $qtS;
						}
					}
				}
				else {
					print "Alignment '$outFile' hast to less member! ==> rejected!\n" if($opt_out==1);
					rejectAln($outFile);
				}
			}

		} # else {	# long enough, write it out
	}
}


# output aln in CLUSTAL format
sub writeClustal {
	print OUT "CLUSTAL W custom format by NcDNAlign\n\n";
					
	my $row=0;
	# Write the aln in aln format ;-)
	for(my $i=0; $i<$trimmedLength; $i += $line_length) {
		my %starSearch;
		$row++;		
			
		my @keys = getSortedKeys();

		foreach my $k(@keys) {
			printf OUT "%-20s", $k;
			printf OUT "%s\n", substr($seq{$k}{trimmedSeq}, $i, $line_length);
			$starSearch{$k}{splitSeq} = $seq{$k}{trimmedSeq}; # for the get_column function
		}
		
		# GIVE ME THE STARS 
		my $empty='';
	
		for(my $j=0; $j<20; $j++) {
			$empty .= ' ';
		}

		print OUT "$empty";
		
		for(my $j=0; $j<$line_length; $j++)
		{
			my $whichColumn = $j+(($row-1) * $line_length);
			my $column = get_column($whichColumn, %starSearch);
			
			last if($column eq '');

			my $firstChar = substr $column, 0, 1;
			if( $column =~ /^$firstChar+$/ ) {
				print OUT "*";
			}
			else {
				print OUT " ";
			}
		}
		print OUT "\n\n";
	}
}

# output aln in FASTA format
sub writeFasta {
	my @keys = getSortedKeys();

	foreach my $k(@keys) {
		printf OUT ">$k\n$seq{$k}{trimmedSeq}\n";
	}
}

# output aln in MAF format
sub writeMAF {
	print OUT "##maf version=1\n";
	print OUT "# NcDNAlign\n";
	print OUT "a score=0.0\n";
	
	my @keys = getSortedKeys();

	foreach my $k(@keys) {
		my $noGaps = $seq{$k}{trimmedSeq};
		$noGaps =~ s/-//g;
		print OUT "s\t$k\t$seq{$k}{absoluteStart}\t".
			length($noGaps)."\t$seq{$k}{strandString}\t1\t$seq{$k}{trimmedSeq}\n";
	}
}


# go through a dialign alignment and parse the sequences 
# by looking at the digitString
# the sequences are split due to a cutoff value of $opt_maxZeros zeros in the digitString.
# flanking zero regions are trimmed automatically
sub parseDigitString {
	my @e = split(/0{$opt_maxZeros,}/, $digitString);
	
	for(my $i=0; $i<=$#e; $i++) { # those are the valid strings
		print "split $i: '$e[$i]' (l=".(length $e[$i]).")\n" if($opt_out==1);
	}
	
	%pos = ();
	
	# we count positions starting with 1 !!!

	my ($curChar, $lastChar, $qtZeros) = ('', '', 0);
	my ($first, $start, $end) 	   = (0, 0, 0);
	
	if($digitString =~/0/) {
		print "parseDigitString:\n" if($opt_out==1);
		for(my $i=0; $i<length $digitString; $i++) {
			$curChar = substr $digitString, $i, 1;
			
			if($curChar ne '0') { # if there is something valid			
				if($first==0){
					$start = $i+1;
					$first=1;					
				}
				$qtZeros = 0;
			}
			else {		
				$qtZeros++;
				
				if($lastChar ne '0' && $qtZeros>$opt_maxZeros) {
					$end = $i;
					$pos{"$start\_$end"}=1 if($start >0 && $end>0);
					$start=0;
					$end=0;
					$first=0; # so that we can get starting psoitions again					
					$qtZeros=0;
				}
				else {					
					if($start>0 && $qtZeros>$opt_maxZeros) { # close it
						$end = $i-$qtZeros+1;
						$pos{"$start\_$end"}=1 if($start >0 && $end>0);
						$start=0;
						$end=0;
						$first=0; # so that we can get starting psoitions again					
						$qtZeros=0;
					}					
				}
			} # else
			
			$lastChar = $curChar;
		} # for
	}
	else {
		print "parseDigitString (no zeros):\n" if($opt_out==1);
		$start = 1;
		$end = (length $digitString);
		$pos{"$start\_$end"}=1;
	}
	
	# get last ending
	if($start>0 && $end==0) {
		$end = (length $digitString) - $qtZeros;
		$pos{"$start\_$end"}=1;
	}
	
	#foreach my $k (sort {$a <=> $b} keys %pos) {
	foreach my $k (sort keys %pos) {
		print "pos = $k\n" if($opt_out==1);
	}
		
	print "\n" if($opt_out==1);
}

# read the original sequence data into the %seq hash
sub read_sequences {
	my ($file) = @_;
	open(IN, "< $file") or die $die_msg."(read sequences): Could not open file '$file'";
	my @filedata = <IN>;
	close(IN);
	
	%seq = ();
	$digitString = '';

	my $seq = '';
	my $id = '';
	my $cnt = 0;

	foreach my $line(@filedata) {
		if($line =~ /^(...\_\d+\S+)\s+\d+\s+(.+)$/) {
			print "$1  $2\n" if($opt_out==1);
			$cnt++;
			$id = $cnt.".".$1;  # To make sure that IDs are unique as aln prog cuts off IDs
			
			if( !exists $seq{$id} ) {
				$seq{$id}{originalSeq}='';
			}
			
			my $subseq = $2;
			$subseq =~ s/\s//g;
			$seq{$id}{originalSeq} .= $subseq;
		}
		elsif($line =~ /^\s*$/)	{
			
			$cnt = 0;       # Alignments are spread over several lines. A blank line indicates 
					# new line of multiple alignment!
			next;
		}
		elsif($line =~ /^\s{23}(\d+)\s*(\d*)\s*(\d*)\s*(\d*)\s*(\d*)/) {
			$digitString .= "$1$2$3$4$5";
			print "\n" if($opt_out==1);
		}
		
	}
	
	# check digitString length and align length
	my $l = length $digitString;
	foreach my $k (keys %seq) {
		die $die_msg."Wrong length! (file='$file', key='$k')\n$seq{$k}{originalSeq}\n$digitString" if( length $seq{$k}{originalSeq} ne $l);
	}
		
	print "$digitString\n\n" if($opt_out==1);
	
	my @Keys = keys %seq;
	
	return length $seq{$Keys[0]}{originalSeq};

}# end of sub read_sequences




sub rejectAln {
	my ($badFile) = @_;
	
	print "Alignment '$badFile' REJECTED, it will be moved to '$badAlnDir'!\n\n" if($opt_s==0);
	
	`mkdir $badAlnDir`if(!(-e $badAlnDir));
			
	`mv $badFile $badAlnDir 2> /dev/null`; # remove error if there is a \d+.2 file and the first already has been moved
}



# trim with decision 'left'
sub trim_withLeftBorder {	
	my ($qtLeft, $qtRight, $maxLeftGapLenID) = @_;
						
	# try dropping the sequence with the highest leftGapLength
	print "\t try dropping $maxLeftGapLenID\n" if($opt_out==1);
	
	# test and maybe switch dropping candidate
	my $looser = switchIfNecessary($maxLeftGapLenID);
	my %tmp = %{$seq{$looser}};
	delete $seq{$looser};
							
	$trimmedLength = getMinOverlap($outFile);
	
	$oldTrimAlnLen = $newTrimAlnLen;
	$newTrimAlnLen = $trimmedLength;

	if($opt_out==1) {
		print "\t new trimmedLength=$trimmedLength, oldLength=$oldTrimAlnLen\n";
	}
	
	# if it is not worthy
	if($oldTrimAlnLen == $newTrimAlnLen) {
		print "\t Trimming will not enlarge the alignment!\n" if($opt_out==1);

		# reactivating
		foreach my $key (keys %tmp) {
			$seq{$looser}{$key} = $tmp{$key}; 
		}
	}
	
	foreach my $k (keys %seq) {
		print "$k   $seq{$k}{trimmedSeq}\n" if($opt_out==1);
	}
	print "\n" if($opt_out==1);
}


# trim with decision 'right'
sub trim_withRightBorder {
	my ($qtLeft, $qtRight, $maxRightGapLenID) = @_;
					
	# try dropping the sequence with the highest rightGapLength
	print "\t try dropping $maxRightGapLenID\n" if($opt_out==1);
	
	# test and maybe switch dropping candidate
	my $looser = switchIfNecessary($maxRightGapLenID);
	my %tmp = %{$seq{$looser}};
	delete $seq{$looser};
	
	$trimmedLength = getMinOverlap($outFile);

	$oldTrimAlnLen = $newTrimAlnLen;
	$newTrimAlnLen = $trimmedLength;

	if($opt_out==1) {
		print "\t new trimmedLength=$trimmedLength, oldLength=$oldTrimAlnLen\n";
	}
	
	# if it is not worthy
	if($oldTrimAlnLen == $newTrimAlnLen) {
		print "\t Trimming will not enlarge the alignment!\n" if($opt_out==1);
		
		# reactivating
		foreach my $key (keys %tmp) {
			$seq{$looser}{$key} = $tmp{$key};
		}
	}
	
	foreach my $k (keys %seq) {
		print "$k   $seq{$k}{trimmedSeq}\n" if($opt_out==1);

	}
	print "\n" if($opt_out==1);
}

# test and maybe switch dropping candidate
# if there is a sequence with more internal gaps, this seq will be dropped first.
# how many internal gaps are allowed?
# not more than $opt_maxGaps
sub switchIfNecessary {
	my ($k) = @_;
	
	my $looser = '';
	
	my $intGapID = getMostInternalGapID();
	
	# invalidate the candidate
	if($intGapID eq $k || $intGapID eq '') {
		$looser = $k; 
	}
	else {	
		print "Switching to '$intGapID' (and dropping), too many internal gaps!\n" if($opt_out==1);
		$looser = $intGapID;
	}

	return $looser;
}


# PURPOSE: Gets the column of an alignment as a string
# PARAMS:  $c: Index of column [0,MAX]
sub get_column {
	my ($c, %hashAlignSeq) = @_;

	my $column = '';

    	my @Keys;
	
	# only use valid sequences
	foreach my $k (sort %hashAlignSeq) {
		if(exists $hashAlignSeq{$k}{splitSeq}) {
			push @Keys, $k;
		}
	}
	
	foreach my $k(@Keys) {
		$column .= substr $hashAlignSeq{$k}{splitSeq}, $c, 1;
	}
	
	return $column;
}# end of sub get_column




# writes trimmed positions to an outfile and collects data for bed file
sub writeTrimFile {

	print TRIMOUT "file=$outFile\n";
	
	my @keys = getSortedKeys();
	
	foreach my $k (@keys) {				
		# bug-compensation:
		$seq{$k}{lTrimmedNucleoLen}=0 if($seq{$k}{lTrimmedNucleoLen}<0);
		$seq{$k}{rTrimmedNucleoLen}=0 if($seq{$k}{rTrimmedNucleoLen}<0);
		
		print TRIMOUT "SEQ|$k| $seq{$k}{lTrimmedNucleoLen} $seq{$k}{rTrimmedNucleoLen}\n";
		
		my $name 		= '';
		my ($start, $end) = (0, 0);
		my $strandString 	= '';
		my $what 		= '';

		my $cmd = '';
															
		if($k =~ /^\d+\.(...\_\d+)[:|\_](\d*)/) {
			$cmd 	 = "grep $1:$2 $fileNumber.fa";
			$name = "$1";					
		}
		else {
			die $die_msg."(1) Wrong key format: k='$k'";
		}
		
		# hitStart and hitEnd are missing
		
		my $ret = `$cmd`;

		if($ret =~ /^>(...\_\d+)\:(\d+)\:(\d+)\:(.)\:hitStart=(\d+)\:hitEnd=(\d+)\:([+|-])/) {
			$what 	= "trimmedAln_$1:$2:$3:$4_file=$floatFileNr";
			
			# Positions start always < than position end !!!			
			$start		= $2 + $5 + $seq{$k}{lTrimmedNucleoLen} -1;
			$end		= $2 + $6 - $seq{$k}{rTrimmedNucleoLen} -1;
			$strandString 	= $7;
			
			# useful for MAF format
			$seq{$k}{strandString}	= $strandString;
			$seq{$k}{absoluteStart}	= $start;
			$seq{$k}{absoluteEnd}	= $end;
			#print STDERR "k='$k', strandString='$seq{$k}{strandString}', aS='$seq{$k}{absoluteStart}, aE='$seq{$k}{absoluteEnd}', end = 2=$2 + 6=$6 - rTrimNucLen=$seq{$k}{rTrimmedNucleoLen} -1;\n";
		}
		else {
			die $die_msg."(2) Return value ='$ret' of sys cmd '$cmd' is not correct! (k=$k)";
		}
				
		$name =~ /^(\S+)\_(\d+)/;
		$bed{$1}{$2}{$start}="$start;$end;$what;$strandString;$name";		
	}
}

# Reject sequences with too many gaps
# by invalidating them at the %seq hash
# returns:	0  if realignment is necessary
#		1  if not
#		-1 if alignment must be rejected
sub gap_score {
	my $changes = 0;
	
	my @keys = getSortedKeys();

	foreach my $k (@keys) {		
		# count all gaps of a trimmed sequence
		my $tmp = $seq{$k}{trimmedSeq};
		$seq{$k}{qtGaps} = $seq{$k}{trimmedSeq}=~ tr/\-//;
								
		my $gap_score = $seq{$k}{qtGaps} / $seq{$k}{trimmedSeqLen};
		
		if( $gap_score >= $C{gap_score}) {
			if($opt_dropRef==1) { 				
				invalidateSeq($k);
				$changes=1 if($changes != -1);				
			}
			else {	# 100 % gaps must be excluded!
				$k =~ /^\d+\.(.+)$/;				
								
				if($k !~ /$C{target_name}/) { # never drop sequence of target organism				
					invalidateSeq($k);
					$changes=1 if($changes != -1);;
				}
				elsif( $gap_score==1 ) {	# except for worst case
					invalidateSeq($k);	# reject alignment
					$changes = -1;	
				}

			}		
		}
		else {
			print "\tGap-Score: $seq{$k}{qtGaps}/$seq{$k}{trimmedSeqLen}=".(nearest(.01, $seq{$k}{qtGaps}/$seq{$k}{trimmedSeqLen}))." < $C{gap_score} ($k)\n" if($opt_out==1);	
		}		
	}	
	
	return $changes;
}


sub invalidateSeq {
	my ($k) = @_;
	
	if($opt_out==1) {
		print "\tGap-Score: ".(nearest(.01, $seq{$k}{qtGaps}/$seq{$k}{trimmedSeqLen}))." >= $C{gap_score}\n";		
		print "\t$k is invalid, it has too many gaps! Realignment necessary!\n";
	}
	
	# save for putative restoring
	foreach my $copyKey (keys %{$seq{$k}} ) {
		$looser{$k}{$copyKey} = $seq{$k}{$copyKey};
	}
		
	delete $seq{$k};	
}



# perform clustalw after dialign, e.g. because RNAz is trained with clustalw
sub performClustalw {
	print "Realignment using clustalw!\n" if($opt_out==1);
	# write sequences to temporary fastafile	
	open(TMPOUT, ">$tmpOutFileFa") or die $die_msg."Can not open tmp file '$tmpOutFileFa'!";
	# write fasta file
	my @keys = getSortedKeys();
	foreach my $k (@keys) {
		$seq{$k}{noGaps} = $seq{$k}{trimmedSeq};
		$seq{$k}{noGaps} =~ s/-//g;
		print TMPOUT ">$k\n$seq{$k}{noGaps}\n";
	}
	close TMPOUT;	
		
	# produce alignments
	my $cmd = "clustalw -INFILE=$tmpOutFileFa -OUTORDER=INPUT -OUTFILE=$outFile 1> /dev/null";
		
	system("$cmd") == 0 or die $die_msg."system call '$cmd' failed: $?";
	unlink $tmpOutFileFa;
	
	# keep tree?
	if($opt_tree == 1) {
		`mv $tmpOutFileDND $outFileDND`;
	}
	else {
		unlink $tmpOutFileDND;
	}
	
	my %hash = readAln($outFile);
	
	my @k = sort keys %hash;
	$trimmedLength = length($hash{$k[0]});
	
	
	if( $trimmedLength>= $C{minAlnLen}) { # valid aln
		$qtAlignments++;
		writeTrimFile(); # after all
		
		if($opt_f =~ /FASTA/) { # read in CLUSTAL and convert to FASTA
			open(TMPOUT, ">$tmpOutFileFa") or die $die_msg."Can not open tmp file '$tmpOutFileFa'!";
			foreach my $k (@k) {
				print TMPOUT ">$k\n$hash{$k}\n";
			}
			close TMPOUT;
			
			system("mv $tmpOutFileFa $outFile") == 0 or die $die_msg."system call '$cmd' failed: $?";
		}
		elsif($opt_f =~ /MAF/) {
			open(TMPOUT, ">$tmpOutFileFa") or die $die_msg."Can not open tmp file '$tmpOutFileFa'!";
			
			print TMPOUT "##maf version=1\n";
			print TMPOUT "# NcDNAlign\n";
			print TMPOUT "a score=0.0\n";
		
			foreach my $k (@k) {
				if($k =~ /^(\d+)\.(\S\S\S\_\d+)(\S+)/ ) {
					my $chrom = $2;
					my $tmp_key = "$1\.$2";
					my $t = $3;
					$t =~ s/\_/\:/g;
					$tmp_key .= $t;
					print TMPOUT "s\t$chrom\t$seq{$tmp_key}{absoluteStart}\t".
						length($seq{$tmp_key}{noGaps})."\t$seq{$tmp_key}{strandString}\t1\t$hash{$k}\n";
				}
				else {
					die $die_msg."Wrong key format: k='$k'";
				}
			}
			close TMPOUT;
			
			system("mv $tmpOutFileFa $outFile") == 0 or die $die_msg."system '$cmd' failed: $?";
		}
		
		my $qtS = scalar(@k);
		print "outfile ==> '$outFile' is ready! (qtSeq='$qtS')\n\n" if($opt_s==0);
	
		if($opt_stat==1) {
			$stat{sumSeqsPerAln} += $qtS;
		}
	}
	else { # alignment too short.
		print "Alignment '$outFile' is too short! (trimmedLength=$trimmedLength)\n" if($opt_s==0);;
		rejectAln($outFile);
		unlink $tmpOutFileDND;
	}
}

# get valid and sorted keys of the seq hash
sub getSortedKeys {
	my @keys;
	foreach my $k (sort keys %seq) {
		push @keys, $k;
	}
	return @keys;
}


# check seq hash
# which seq has the highest number of internal gaps?
# internal gaps are the gaps of the trimmed sequence
# but we only start counting them if $opt_maxGaps is reached 
sub getMostInternalGapID {
	my @keys = getSortedKeys();
	my $winnerID = '';
	my $count = 0;
	foreach my $k (@keys) {
		if($opt_dropRef==1) {
			my $curCount = $seq{$k}{trimmedSeq} =~ tr/-//;
			if($curCount>=$opt_maxGaps && $curCount>$count) {
				$winnerID=$k;
				$count=$curCount;
			}
		}
		else {
			if($k !~ /$C{target_name}/) { # never drop sequence of target organism
				my $curCount = $seq{$k}{trimmedSeq} =~ tr/-//;
				if($curCount>=$opt_maxGaps && $curCount>$count) {
					$winnerID=$k;
					$count=$curCount;
				}
			}				
		}		
	}
	
	print "\t most internal gaps: '$winnerID' (qtGaps=$count)\n" if($opt_out==1);
	
	return $winnerID;
}



# who has the longest? (in terms of gapLen, not what you are thinking about *g*)
sub getMaxGapLen {
	my ($k) = @_;
	
	# better value || initiate
	if( $seq{$k}{lGapsLen} > $maxLeftGapLen || $maxLeftGapLenID eq 0) {
		# new max
		$maxLeftGapLen	 = $seq{$k}{lGapsLen};
		$maxLeftGapLenID = $k;
	}
	
	if( $seq{$k}{rGapsLen} > $maxRightGapLen || $maxRightGapLenID eq 0) {
		# new max
		$maxRightGapLen	  = $seq{$k}{rGapsLen};
		$maxRightGapLenID = $k;
	}
}


__END__

=head1 NAME

C<trimAln.pl> - part of the NcDNAlign alignment pipeline, step (4)

Remove flanking gaps (trim leading and succeeding gaps and N's) of all *.aln files in the alignment directory. Beautify the alignments by length maximization.

=head1 SYNOPSIS

realign.pl [options]

=head1 OPTIONS

=over 8

=item B<-c, --conf> FILE

Path to central NcDNAlign configuration file [REQUIRED]

=item B<-o, --out> [0|1]

Printing detailed results at STDOUT ON(1) or OFF(0). Default: 0.

=item B<-g, --maxGaps> INT

The minimal number of connected gaps enforcing application of the beautification algorithm. The maximal allowed number of connected gaps in a sequence is maxGaps-1. Default is 120, an appropriate window size for third party programs (e.g. RNAz).

=item B<-z, --maxZeros> INT

The minimal number of connected zeros enforcing a DIALIGN alignment to be splitted. The maximal allowed number of connected zeros in the DIALIGN alignment score-string (sum-of-weights indicates the degree of local similarity among sequences) is maxZeros-1. Default is 20, an appropriate value to split one alignment into two.

=item B<-d, --dropRef> [0|1]

Is it allowed to drop the reference sequence out of an alignment for beautification purposes? 0=NO (default), 1=YES 

=item B<-s, --silent> [0|1]

Silent mode, avoid printing to STDOUT
0=OFF (Default), 1=ON

=item B<--stat> [0|1]

Calculate statistics OFF(0, default) or ON(1). Statistics are printed to STDOUT and into the 'aln.statistics' file. Not fully implemented yet.

=item B<-r, --realign> [0|1]

After all, should a realignment step using CLUSTALW be performed? 1=YES (default), 0=NO 

=item B<-t, --tree> [0|1]

If realignment step using CLUSTALW is performed, should we store the *.dnd tree file? 0=NO (default), 1=YES 
In case of large-screen analyses there could be many files and the tree info is not always needed. 

=item B<-f, --format> [CLUSTAL|FASTA|MAF]

Output alignment format. Default: CLUSTAL

=item B<-v, --version>

Prints version information and exits.

=item B<-h, --help>

Prints a short help message and exits.

=item B<--man>

Prints a detailed manual page and exits.

=back

=head1 DESCRIPTION

Regardless of the applied alignment algorithm, MSAs consisting of pairwise alignments may need beautification, conveniently, because of too large variations in sequence lengths. DIALIGN2-2 prints out a sum-of-weight-score indicating the degree of local similarity among sequences for each alignment column. However, we use this score to split one alignment into two if at least --maxZeros consecutive zeros are read in to separate valid local alignments from non-alignable regions ($x=20$ could be an appropriate default value). Moreover, we test the alignments (1) if their length exceeds the minimal length (Minimal length of the overlap is the same value as for retaining the local alignments in the configuration file) and (2) if they contain --maxGaps consecutive gaps (120 could be a valuable threshold for many alignment processing tools applying sliding windows). Obviously, it is useless to scan sequences that contain more consecutive gaps than the length of the sliding window. If (1) or (2) is true, the beautification algorithm is applied to the alignment until the number of aligned sequences exceeds the minimal number of species in the screen or no improvement is achieved. Trimmed/Improved alignment files are outputted and a BED file 'trimmedAln.bed' for the coordinates of the trimmed alignments (not every alignment format handles coordinates) is created.

=head1 EXAMPLES

$ trimAln.pl -c config-file.cfg

=head1 AUTHORS

Dominic Rose (dominic@bioinf.uni-leipzig.de)

=head1 AVAILABILITY

http://www.bioinf.uni-leipzig.de/Software/NcDNAlign/

=cut




